/*
 * **********************************
 *                                   *
 * HLC Games - Software Entertainment *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC BrickCoverage - (1.6.2010) ************************
 *
 * (c) Copyright 2010 - Lukas Hermanns
 *
 * Basic brick header
 *
 */

#ifndef __BC_BASICBRICK_H__
#define __BC_BASICBRICK_H__


#include "bcStandard.hpp"


namespace bc
{


enum EBasicBricks
{
    BRICK_1_1_1,
    BRICK_1_3_1,
    BRICK_1_1_2,
    BRICK_1_3_2,
    BRICK_2_1_2,
    BRICK_2_3_2,
    BRICK_3_1_2,
    BRICK_3_3_2,
    BRICK_4_1_2,
    BRICK_4_3_2,
    
    BRICK_EXTENDED,
    
    BRICK_COUNT
};

enum EBrickColors
{
    COLOR_WHITE,
    COLOR_BLACK,
    COLOR_RED,
    COLOR_GREEN,
    COLOR_BLUE,
    COLOR_YELLOW,
    COLOR_ORANGE,
    COLOR_GRAY,
    COLOR_BROWN,
    COLOR_GLASS,
    
    COLOR_EXTENDED,
    
    COLOR_COUNT,
};


class BasicBrick
{
    
    public:
        
        BasicBrick(
            scene::Mesh* BrickMesh, const EBasicBricks BasicType, const dim::vector3di &Size, const dim::aabbox3df &BoundBox
        );
        virtual ~BasicBrick();
        
        /* Basic functions */
        
        virtual void Drag();
        virtual void Drop();
        
        virtual void SetPosition(const dim::vector3df &Position);
        virtual void SetPosition(const dim::vector3di &Position);
        
        virtual void SetRotation(s32 Angle);
        
        virtual void TurnCCW();
        virtual void TurnCW();
        
        virtual void Move(s32 Horz, s32 Vert);
        
        virtual void SetColorType(const EBrickColors Type);
        virtual void SetColor(const video::color &Color);
        
        virtual void SetBasicBrickType(const EBasicBricks Type);
        
        virtual void Update();
        
        virtual void DrawBoundBox(const video::color &Color = video::color(0, 0, 255));
        
        /* Static functions */
        
        static void LoadBasicBricks();
        static BasicBrick* Create(const EBasicBricks Type, bool isLoaded = false);
        
        static void UpdateBricks();
        
        static void DrawMenuCurrentBrick();
        static bool DrawMenuColorPalette(EBrickColors &ChoosenColor);
        
        /* Inline functions */
        
        inline void SetActive(bool Enable)
        {
            Active_ = Enable;
        }
        inline bool GetActive() const
        {
            return Active_;
        }
        
        inline EBasicBricks GetBasicType() const
        {
            return BasicType_;
        }
        inline scene::Mesh* GetMesh() const
        {
            return Mesh_;
        }
        inline dim::vector3di GetPosition() const
        {
            return Position_;
        }
        inline dim::vector3di GetSize() const
        {
            return Size_;
        }
        inline EBrickColors GetColorType() const
        {
            return ColorType_;
        }
        inline video::color GetColor() const
        {
            return Color_;
        }
        inline bool IsInteractive() const
        {
            return Interactive_;
        }
        
    protected:
        
        friend bool SaveMap(const io::stringc &Filename);
        friend bool LoadMap(const io::stringc &Filename);
        
        static const dim::vector3df BRICK_DIMENSION;
        
        /* Functions */
        
        void InitMesh();
        
        /* Members */
        
        scene::Mesh* Mesh_;
        scene::SPickingObject* PickObj_;
        EBasicBricks BasicType_;
        
        /**
        Width = 1, Height = 1, Depth = 1 is the smallest brick.
        A height of 3 is a normal value.
        */
        dim::vector3di Size_;
        dim::vector3di Position_;
        s32 Rotation_;
        dim::aabbox3df BoundBox_;
        
        EBrickColors ColorType_;
        video::color Color_;
        
        bool Interactive_;
        
        bool Active_;
        
};

extern BasicBrick* CurDragedBrick;
extern std::list<BasicBrick*> BrickObjectList;


} // /namespace bc


#endif








// ================================================================================
